@echo off
setlocal
setlocal enabledelayedexpansion

REM Authored by Colin Caine 2014
REM I hereby place this work in the public domain.

set managerdir=modman\
set sourcedir=modman_install_files\

echo.
echo ### modman v1.4 installer ###
echo.
echo This script creates a new modman directory and copies everything in the
echo current directory (excluding the files that come with this script^) into
echo .\%managerdir%_original.
echo.
echo modman's own files scripts will be placed in .\%managerdir%
echo.
echo This script will also offer to install kyrub and yitzi's patches and will 
echo perform a first time initialisation
echo.
echo After install, this script will self destruct.
echo.
echo Consult %managerdir%modman_readme.txt for further advice and usage information.
echo.


fsutil >nul
if ERRORLEVEL 1 (
  echo I need adminisitrator privileges to create hardlinks.
  echo Please run me as administrator.
  echo.
  PAUSE
  goto :eof
)

choice /M "Do you want to install me now?"
if %ERRORLEVEL% NEQ 1 goto :error
echo.

REM Change to the directory this batch script is in.
cd /d "%~dp0"

REM Check if we're already installed.
if exist %managerdir% (
  echo %managerdir% already exists^^!
  echo.
  echo Looks like mod_manager is already installed. Installing again will overwrite
  echo _original, manage.bat and order.txt.
  choice /M "Do you want to install anyway?"
  if !ERRORLEVEL! NEQ 1 goto :error
  echo.
)

set dest=%managerdir%_original
mkdir %dest% 2>nul

echo Moving existing files and directories (allow some time^)...

for %%F in (*) do (
  if "%%F" NEQ "%~nx0" (
    if "%%F" NEQ "modman_readme.txt" (
      move "%%F" "%dest%"
      if errorlevel 1 (
        echo Could not move "%%F" into "%dest%".
        goto :error
      )
    )
  )
)

for /d %%F in (*) do (
  if "%%F\" NEQ "%managerdir%" (
    if "%%F\" NEQ "%sourcedir%" (
      move "%%F" "%dest%"
    )
  )
)

echo.
echo Moving modman files...
echo.

REM Hide errorlog from modman_readme in case they don't extract the readme
REM for install -- don't want to scare them with an error message from move.
move "modman_readme.txt" "%managerdir%" 2>nul
move "%sourcedir%\*" "%managerdir%"
for /d %%F in ("%sourcedir%\*") do (
  move "%%F" "%managerdir%"
)
rmdir %sourcedir%

echo.
echo Modman installed^^!
echo.
echo Your game files will now have disappeared, don't worry, though, they'll
echo be back in a moment.
echo.

if EXIST %dest%\terran.exe (
  echo Looks like this was a SMAC install. Moving the save directory back out of
  echo _original (trust me, this is a good idea^).
  move %dest%\saves saves

  echo.
  choice /M "Do you want to enable the included copies of kyrub and yitzi's patches?"
  if !ERRORLEVEL!==1 (
    echo yitzi_2.5 >> %managerdir%order.txt
    echo yitzi_2.5e >> %managerdir%order.txt
    echo kyrub_444_n >> %managerdir%order.txt
  ) else (
    echo #yitzi_2.5 >> %managerdir%order.txt
    echo #yitzi_2.5e >> %managerdir%order.txt
    echo #kyrub_444_n >> %managerdir%order.txt
  )
)

echo.
echo Righto. Performing initial manage.bat...
echo.

call %managerdir%manage.bat -f

echo.
echo The game should now be installed. For more details, read 
echo %managerdir%modman_readme.txt.
echo.
echo I will now self destruct. Bye bye!

PAUSE
del %0

goto :eof

:error
echo modman was not installed.
PAUSE
goto :eof
