@echo off

REM Authored by Colin Caine 2014
REM I hereby place this work in the public domain.

setlocal
setlocal enableextensions
setlocal enabledelayedexpansion

fsutil >nul
if ERRORLEVEL 1 (
  echo I need adminisitrator privileges to create hardlinks.
  echo Please run me as administrator.
  echo.
  PAUSE
  goto :eof
)

REM Change to the directory this batch script is in.
cd /d "%~dp0"

REM Paths relative to this batch file.
set rootdir=..\
set managerdir=
set order=%managerdir%order.txt

REM The fully qualified name of this directory.
set fqmanagerdir=%CD%\

REM The fully qualified name of the root directory.
pushd %rootdir%
set fqrootdir=%CD%\
popd

REM Do some funky stuff to get just the name of the directory we're
REM in. Used to stop mods making us delete ourselves and so on.
call :get_managerdir_name "%fqmanagerdir%" managerdirname



REM
REM Interactive bit.
REM

echo.
echo ### modman v1.4 ###

echo.
echo Known mods:
for /d %%F in ("%managerdir%*") do (
  echo.  %%F
)

echo.
echo Mods to enable (in order):
for /F "usebackq delims= eol=#" %%F in ("%order%") do (
  echo.  %%F
)

echo.
echo (Edit %order% in a text editor to change mod order and to enable
echo or disable mods)

echo.
echo (Read modman_readme.txt for more information)

REM If called with -f, skip the question.
if "%1"=="-f" goto :nochoice

echo.
choice /M "Do you want to enable these mods?"
if %ERRORLEVEL% NEQ 1 (
  echo Very well.
  PAUSE
  goto :eof
)


:nochoice

REM
REM Rebuild the game directory.
REM

echo.
echo Removing links to known mods...

for /d %%N in ("%managerdir%*") do (
  REM Delete the links for all mods. This function checks for non-link files
  REM and will cause the program to exit if it is trying to delete one.
  REM set _skip=0
  REM findstr /b /l "%%N" %order% >nul && set _skip=1
  REM if !_skip!==1 (
  REM   echo.  Skipping %%N...
  REM ) else (
    echo.  Deleting links to %%N...
    call :del_links "%%N" "%fqrootdir%" "%fqmanagerdir%"
  REM )
)

echo.
echo Pruning empty directory trees...

call :prune_dir_tree "%rootdir%"
echo.  Pruned.

echo.
echo Installing selected mods...

for /F "usebackq delims= eol=#" %%N in ("%order%") do (
  echo.  Installing %%N...
  if not exist "%%N" (
    echo.    Mod directory does not exist^^!
  ) else (
    call :directories %%N %rootdir%
    call :files %%N %rootdir%
  )
)

echo.
echo Done^^!
echo.
PAUSE
goto :eof



REM
REM Functions follow...
REM

:get_managerdir_name	--Kludge to get the name of this directory
set "%~2=%~n1"
goto :eof

:directories		--Makes all the directories for a mod
for /r %~1 %%M in (.) do (
  set _partial=%%~fM
  set _path=!_partial:*%1=!
  mkdir "%2!_path!" 2>nul
)
goto :eof

:files			--Links in all the required files
REM Use mklink if it exists (it's much faster)
mklink /H 2>nul

REM ERRORLEVEL=9009 on win7 after entering a command that cmd doesn't
REM recognize.
if ERRORLEVEL 2 (
  call :files_fsutil %1 %2
) else (
  call :files_mklink %1 %2
)
goto :eof

:files_fsutil		--Link with fsutil (slow)
set count=0
set total=0

for /r %~1 %%M in (*) do (
  set _partial=%%~fM
  set _path=%2!_partial:*%1\=!

  REM Warning: allows users to write to the mod manager directory
  if exist "!_path!" del "!_path!" 2>nul
  fsutil hardlink create "!_path!" "%%M" 1>nul

  REM Show that we're doing something
  set /a count+=1
  set /a count_mod=count / 100
  if !count_mod! GEQ 1 (
    set /a total=total+count
    set /a count=0
    echo.    !total! links created...
  )
)
goto :eof


:files_mklink		--Link with mklink (fast, vista or later)
set count=0
set total=0

for /r %~1 %%M in (*) do (
  set _partial=%%~fM
  set _path=%2!_partial:*%1\=!

  REM Warning: allows users to write to the mod manager directory
  if exist "!_path!" del "!_path!" 2>nul
  mklink /H "!_path!" "%%M" 1>nul

  REM Show that we're doing something
  set /a count+=1
  set /a count_mod=count / 200
  if !count_mod! GEQ 1 (
    set /a total=total+count
    set /a count=0
    echo.    !total! links created...
  )
)
goto :eof


:del_links		--Deletes all links from a mod
set count=0
set total=0

for /r %~1 %%M in (*) do ( 
  REM Strips off the leading path to the mod directory.
  set _partial=%%~fM
  set _path=!_partial:*%~1\=!
  set _fqpath=%~2!_path!
  
  REM Warning: allows users to delete files in the mod manager directory.
  REM Check if a file has any hardlinks before deleting it. Don't delete a file
  REM with no hardlinks.
  if exist "!_fqpath!" (
    set _skipped=1
    fsutil hardlink list "!_fqpath!" | find "%~p3" >nul ^
      && set _skipped=0 && del "!_fqpath!"
    if !_skipped!==1 (
      echo.
      echo.    "!_fqpath!" not a hardlink^^!
      echo.    File not deleted.
      echo.    File "!_path!" in mods will not be linked in.
      echo.
      echo.    If this is a save file or some other file that the game
      echo.    generates you should consider moving the save file directory
      echo.    from mod %1 to the root game directory.
      echo.
      echo.    Ultimately, you must do one of the following:
      echo.      1. Remove this file from the game directory
      echo.           (it will be replaced by a link to a file of the same name in
      echo.            one of your enabled mods^)
      echo.      2. Rename this file in either the mod or the game directory
      echo.           (both versions will be kept^)
      echo.      3. Remove this file in all mod directories (including disabled mods^)
      echo.           (the current file in the game directory will be left in place^)
      echo.      4. Move this file to a new mod directory and enable that mod
      echo.           (the file will be linked into the game directory as with other
      echo.            mods.^)
      echo.
      goto :error
    )
    set /a count+=1
  )

  REM Show that we're doing something
  set /a count_mod=count / 100
  if !count_mod! GEQ 1 (
    set /a total=total+count
    set /a count=0
    echo.    !total! links removed...
  )
)
goto :eof


:dangerous_del_links	--Don't bother checking if a file is a link or not.
set count=0
set total=0
for /r %~1 %%M in (*) do (
  
  REM Strips off the leading path to the mod directory.
  set _partial=%%~fM
  set _path=!_partial:*%~1\=!
  
  REM Warning: allows users to delete files in the mod manager directory.
  REM Don't check if a file has any hardlinks before deleting it.
  if exist "%~2!_path!" (
    del "%~2!_path!" 2>nul
    set /a count+=1
  )

  REM Show that we're doing something.  
  set /a count_mod=count / 200
  if !count_mod! GEQ 1 (
    set /a total=total+count
    set /a count=0
    echo.    !total! links removed...
  )
)
goto :eof

:prune_dir_tree		--Prunes a directory tree of empty branches.
REM Attempt to remove each directory in depth first order.
REM Removal attempt is safe as rmdir will not remove a non-empty folder
REM unless forced.

pushd "%1"
for /d %%M in (*) do (
  if "%%M" NEQ "%managerdirname%" (
    call :prune_dir_tree "%%M"
    rmdir "!CD!\%%M" 2>nul
  )
)
popd
goto :eof

:error
echo.
echo Update not complete.
PAUSE
exit
